"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
class Encode {
    utob(u) {
        return u.replace(/[\uD800-\uDBFF][\uDC00-\uDFFFF]|[^\x00-\x7F]/g, (c) => {
            if (c.length < 2) {
                let cc = c.charCodeAt(0);
                return cc < 0x80 ? c
                    : cc < 0x800 ? (util_1.default.fromCC(0xc0 | (cc >>> 6))
                        + util_1.default.fromCC(0x80 | (cc & 0x3f)))
                        : (util_1.default.fromCC(0xe0 | ((cc >>> 12) & 0x0f))
                            + util_1.default.fromCC(0x80 | ((cc >>> 6) & 0x3f))
                            + util_1.default.fromCC(0x80 | (cc & 0x3f)));
            }
            else {
                let cc = 0x10000
                    + (c.charCodeAt(0) - 0xD800) * 0x400
                    + (c.charCodeAt(1) - 0xDC00);
                return (util_1.default.fromCC(0xf0 | ((cc >>> 18) & 0x07))
                    + util_1.default.fromCC(0x80 | ((cc >>> 12) & 0x3f))
                    + util_1.default.fromCC(0x80 | ((cc >>> 6) & 0x3f))
                    + util_1.default.fromCC(0x80 | (cc & 0x3f)));
            }
        });
    }
    btoa(bin, ch, padchar) {
        let u32, c0, c1, c2, asc = '';
        let { chs } = util_1.default.getChsTab(ch);
        const pad = bin.length % 3;
        for (let i = 0; i < bin.length;) {
            if ((c0 = bin.charCodeAt(i++)) > 255 ||
                (c1 = bin.charCodeAt(i++)) > 255 ||
                (c2 = bin.charCodeAt(i++)) > 255)
                throw new TypeError('invalid character found');
            u32 = (c0 << 16) | (c1 << 8) | c2;
            asc += chs[u32 >> 18 & 63]
                + chs[u32 >> 12 & 63]
                + chs[u32 >> 6 & 63]
                + chs[u32 & 63];
        }
        return pad ? asc.slice(0, pad - 3) + (padchar || "===").substring(pad) : asc;
    }
    get(bin, ch, padchar) {
        return this.btoa(this.utob(bin), ch, padchar);
    }
}
exports.default = new Encode();
